#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.

from unittest import TestCase

from liminal.core.config.defaults import default_configs


class TestDefaultsServiceConfig(TestCase):
    def test_apply(self):
        subliminal = {
            "services": [
                {
                    "name": "my_python_server",
                    "type": "python_server",
                    "image": "default_image"
                },
                {
                    "name": "my_python_server_for_stg",
                    "type": "python_server",
                    "image": "test_image"
                }
            ]}

        superliminal = {
            "service_defaults": {
                "param": "param1",
                "param2": "param2"
            }
        }

        expected = [{'image': 'default_image',
                     'name': 'my_python_server',
                     'param': 'param1',
                     'param2': 'param2',
                     'type': 'python_server'},
                    {'image': 'test_image',
                     'name': 'my_python_server_for_stg',
                     'param': 'param1',
                     'param2': 'param2',
                     'type': 'python_server'}]

        self.assertEqual(expected, default_configs.apply_service_defaults(subliminal, superliminal))
