#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.

from liminal.runners.airflow.tasks import containerable


class PythonTask(containerable.ContainerTask):
    """
    Python task.
    """

    def _kubernetes_cmds_and_arguments(self, output_path, output_destination_path):
        cmds = ['/bin/bash', '-c']
        arguments = [

            self.__cmd(output_path, output_destination_path)
        ]

        return cmds, arguments

    def __cmd(self, output_path, output_destination_path):
        return self.task_config['cmd']
